var http = require("http");
var querystring = require("querystring");

function parseResponse(response, resolve, reject){
	var data = "";
	if(response.statusCode >= 200 && response.statusCode < 300){
		response.setEncoding('utf8');
		response.on('data', function(chunk){
			data += chunk;
		});
		response.on('end', function(){
			try{
				data = JSON.parse(data);
				resolve(data);
			}catch(e){
				reject(e);
			}
		});
	}else{
		reject(new Error(`http code: ${response.statusCode}`));
	}
}

module.exports = {
	// 连接一个库
	db: function(host){
		host = host.split(":");
		var hostname = host[0];
		var port = host[1] || "80";
		host = host.join(":");

		return {
			// 打开一张表
			useTable: function(tableName){
				return {
					// 查询数据
					select: function(columns, where){
						where = JSON.stringify(where);

						return new Promise(function(resolve, reject){
							http.get(`http://${host}/${tableName}?columns=${columns}&where=${where}`, function(response){
								parseResponse(response, resolve, reject);
							});
						});
					},
					// 插入一条数据
					insert: function(values){
						return new Promise(function(resolve, reject){
							var postData = querystring.stringify({
								values: JSON.stringify(values)
							});

							var request = http.request({
								host: hostname,
								port: port,
								path: "/" + tableName,
								method: "POST"
							}, function(response){
								parseResponse(response, resolve, reject);
							});

							request.write(postData);
							request.end();
						});
					},
					// 更新数据
					update: function(sets, where){
						return new Promise(function(resolve, reject){
							var putData = querystring.stringify({
								sets: JSON.stringify(sets),
								where: JSON.stringify(where)
							});

							var request = http.request({
								host: hostname,
								port: port,
								path: "/" + tableName,
								method: "PUT"
							}, function(response){
								parseResponse(response, resolve, reject);
							});

							request.write(putData);
							request.end();
						});
					},
					// 删除数据
					delete: function(where){
						where = JSON.stringify(where);

						return new Promise(function(resolve, reject){
							var request = http.request({
								host: hostname,
								port: port,
								path: `/${tableName}?where=${where}`,
								method: "DELETE"
							}, function(response){
								parseResponse(response, resolve, reject);
							});

							request.end();
						});
					}
				};
			},
			// 创建一张表
			createTable: function(tableName, columns){
				return new Promise(function(resolve, reject){
					var postData = querystring.stringify({
						tableName: tableName,
						columns: JSON.stringify(columns)
					});

					var request = http.request({
						host: hostname,
						port: port,
						method: "POST"
					}, function(response){
						parseResponse(response, resolve, reject);
					});

					request.write(postData);
					request.end();
				});
			},
			deleteTable: function(tableName){},
			// 断开数据库
			close: function(){}
		};
	}
};