var http = require('http');
var fs = require("fs");
var path = require("path");
var Url = require("url");
var QueryString = require('querystring');
var config = require("../config.json");
var getDeps = require("./get-deps");
var getMods = require("./get-mods");
var publish = require("./publish");
var publishFiles = require("./publish-files");
var filesPath = require("./files-path");
var Mime = require("mime");

function readRequestData(request){
	return new Promise(function(resolve, reject){
		var data = "";
		request.setEncoding('utf8');
		request.on('data', function(chunk){
			data += chunk;
		});
		request.on('end', function(){
			try{
				data = JSON.parse(data);
				resolve(data);
			}catch(e){
				reject(e);
			}
		});
	});
}

var server = http.createServer(function (req, res) {
	var url = Url.parse(req.url);
	var pathname = url.pathname;
	var querystring = QueryString.parse(url.query);

	console.log(req.url);

	switch(pathname){
		case "/deps":
			getDeps(querystring.mod).then(function(modIds){
				res.writeHead(200, {
					'Content-Type': 'text/plain'
				});
				res.write(JSON.stringify({
					code: 200,
					files: modIds.split(",")
				}));
				res.end();
			});
			break;
		case "/deps-jsonp":
			getDeps(querystring.mod).then(function(modIds){
				res.writeHead(200, {
					'Content-Type': 'text/plain'
				});
				// res.write(modIds);
				res.write(`loader.jsonp["${querystring.mod}"]("${modIds}")`);
				res.end();
			});
			break;
		case "/mods":
			getMods(querystring.ids).then(function(content){
				res.writeHead(200, {
					'Content-Type': 'application/x-javascript'
				});
				res.write(content);
				res.end();
			});
			break;
		case "/publish":
			readRequestData(req).then(function(info){
				publish(info).then(function(result){
					res.writeHead(200, {
						'Content-Type': 'application/json'
					});
					res.write(JSON.stringify(result));
					res.end();
				});
			});
			break;
		case "/publish-files":
			readRequestData(req).then(function(info){
				publishFiles(info).then(function(){
					res.writeHead(200, {
						'Content-Type': 'application/json'
					});
					res.end();
				});
			});
			break;
		default:
			if(/^\/static/.test(pathname)){
				fs.readFile(path.join(filesPath, "assets", pathname.replace(/^\/static\//, "")), null, function(err, content){
					if(err){
						console.error(err);
						res.writeHead(500, {
							'Content-Type': 'text/html'
						});
						res.write(`<p>${err.message}</p>`);
						res.end();
						return;
					}

					res.writeHead(200, {
						'Content-Type': Mime.lookup(pathname) || "text/panel"
					});
					res.write(content);
					res.end();
				});
			}else{
				res.writeHead(404, {
					'Content-Type': 'text/html'
				});
				res.write(`<p>服务不存在</p>`);
				res.end();
			}
	}
});

module.exports = {
	listen: function(port){
		server.listen(port);
		console.log("static server run at port " + port);
	}
}