var DBClient = require("../db/client");
var utils = require("node-pearls");
var config = require("../config.json");

var hexIndex = "0123456789abcdefghijklmnopqrstuvwxyz";
function parseInt(num){
	var result = 0;
	var hex = hexIndex.length;

	for(var i = 0, l = num.length; i < l; i ++){
		result += hexIndex.indexOf(num.charAt(i)) * Math.pow(hex, l - i - 1);
	}

	return result;
}

module.exports = function(info){
	var db = DBClient.db(config.db.host + ":" + config.db.port);
	var depsTable = db.useTable("deps");
	var depsAsyncTable = db.useTable("depsAsync");
	var abbrTable = db.useTable("abbr");
	var md5Table = db.useTable("md5");

	return info.files.reduce(function(pre, item){
		return pre.then(function(results){
			return md5Table.select("last(abbr)", {
				md5: item.md5
			}).then(function(md5Result){
				if(md5Result){
					return abbrTable.select("last(abbr)", {
						modId: item.file
					}).then(function(abbrResult){
						if(abbrResult && abbrResult.abbr === md5Result.abbr){
							return results;
						}else{
							return abbrTable.insert({
								modId: item.file,
								abbr: md5Result.abbr
							}).then(function(){
								return results;
							});
						}
					});
				}else{
					return md5Table.select("last(abbr)").then(function(result){
						var abbr = 0;
						if(result){
							abbr = parseInt(result.abbr) + 1;
						}
						abbr = utils.hex(abbr);

						return Promise.all([
							md5Table.insert({
								abbr: abbr,
								md5: item.md5
							}),
							abbrTable.insert({
								abbr: abbr,
								modId: item.file
							})
						].concat((item.deps || []).map(function(dep){
							return depsTable.insert({
								abbr: abbr,
								depModId: dep
							});
						})).concat((item.asyncDeps || []).map(function(dep){
							return depsAsyncTable.insert({
								abbr: abbr,
								depModId: dep
							});
						}))).then(function(){
							return results.concat({
								file: item.file,
								abbr: abbr
							});
						});
					});
				}
			});
		});
	}, Promise.resolve([]));
};